/******************************************************************************
 *
 * 文件名  ： running.c
 * 负责人  ： 彭鹏(pengpeng@fiberhome.com)
 * 创建日期： 20151211 
 * 版本号  ： v1.0
 * 文件描述： 当前运行任务实现
 * 版权说明： Copyright (c) 2000-2020 GNU
 * 其    他： 
 * 修改日志： 无
 *
 *******************************************************************************/


/*---------------------------------- 预处理区 ---------------------------------*/

/************************************ 头文件 ***********************************/
#include "cmos_config.h"
#include "tcb.h"
#include "running.h"

/*----------------------------------- 声明区 ----------------------------------*/

/********************************** 变量声明区 *********************************/

/********************************** 函数声明区 *********************************/

/********************************** 变量实现区 *********************************/
cmos_task_tcb_T *s_running_tcb = NULL;

/********************************** 函数实现区 *********************************/
/*******************************************************************************
 *
 * 函数名  : cmos_task_pool_running_set_tcb
 * 负责人  : 彭鹏
 * 创建日期：20151213 
 * 函数功能: 设置当前运行任务
 *
 * 输入参数: tcb 当前运行任务tcb
 * 输出参数: 无
 * 返回值  : 无
 *          
 * 调用关系: 无
 * 其 它   : 无
 *
 ******************************************************************************/
inline void cmos_task_pool_running_set_tcb(cmos_task_tcb_T *tcb)
{
    s_running_tcb = tcb;
}

/*******************************************************************************
 *
 * 函数名  : cmos_task_pool_running_get_tcb
 * 负责人  : 彭鹏
 * 创建日期：20151214 
 * 函数功能: 获取当前运行任务
 *
 * 输入参数: 无
 * 输出参数: 无
 * 返回值  : 当前运行任务
 *          
 * 调用关系: 无
 * 其 它   : 无
 *
 ******************************************************************************/
inline cmos_task_tcb_T *cmos_task_pool_running_get_tcb(void)
{
    return s_running_tcb;
}

/*******************************************************************************
 *
 * 函数名  : cmos_task_pool_running_set_delay
 * 负责人  : 彭鹏
 * 创建日期：20151213 
 * 函数功能: 设置当前运行的延迟
 *
 * 输入参数: tcb 当前运行任务tcb
 * 输出参数: 无
 * 返回值  : 无
 *          
 * 调用关系: 无
 * 其 它   : 无
 *
 ******************************************************************************/
inline void cmos_task_pool_running_set_delay(cmos_int32_T millisec)
{
    cmos_task_tcb_set_delay_ms(s_running_tcb, millisec);
}

/*******************************************************************************
 *
 * 函数名  : cmos_task_pool_running_set_psp
 * 负责人  : 彭鹏
 * 创建日期：20151213 
 * 函数功能: 保存当前运行任务的psp
 *
 * 输入参数: psp 当前运行任务psp
 * 输出参数: 无
 * 返回值  : 无
 *          
 * 调用关系: 无
 * 其 它   : 无
 *
 ******************************************************************************/
inline void cmos_task_pool_running_set_psp(cmos_task_tcb_psp_T psp)
{ 
    cmos_task_tcb_set_psp(s_running_tcb, psp);
}

/*******************************************************************************
 *
 * 函数名  : cmos_task_pool_running_update_tick
 * 负责人  : 彭鹏
 * 创建日期：20151214 
 * 函数功能: 更新当前运行任务的tick
 *
 * 输入参数: 无
 * 输出参数: 无
 * 返回值  : 无
 *          
 * 调用关系: 无
 * 其 它   : 无
 *
 ******************************************************************************/
inline void cmos_task_pool_running_update_tick(void)
{
    cmos_task_tcb_dec_tick(s_running_tcb);
}

